#include "maibu_res.h"
#include "maibu_sdk.h"

//全局变量
static int32_t g_window = -1;
static P_Window init_watch();
static char week[7][30] = {0}; //每周字符串
static int16_t yearpmon;       //日历每月更新一次
static int8_t mode_switch = 4; //切换模式
static int8_t mon_days;        //月总天数
static int8_t first_day;       //月第一天是周几
//计算本月天数
static int8_t get_mon_days(int16_t year, int8_t mon)
{
  int8_t m[13] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
  if ((year % 4 == 0 && year % 100 != 0) || year % 400 == 0)
    m[2]++;
  int8_t result = m[mon];
  return result;
}
//本月第一天周几
static int8_t get_first_day(int8_t day, int8_t wday)
{
  int8_t result = wday - (day - 1) % 7;
  if (result < 0)
    result += 7;
  return result;
}
//生成日历字串
static void calendar_str(int8_t first_day, int8_t mon_days)
{
  int8_t i = 1, j = 1, k = 1;
  char tmp[5] = {0};
  memset(week, 0, sizeof(week));
  //第1行
  for (i = 1; i <= 7 - first_day; i++)
  {
    sprintf(tmp, "%02d ", j);
    strcat(week[1], tmp);
    j++;
  }
  week[1][strlen(week[1]) - 1] = '\0';
  //第2到6行
  for (k = 2; k <= 6; k++)
  {
    if (j <= mon_days)
    {
      memset(tmp, 0, sizeof(tmp));
      for (i = 1; i <= 7 && j <= mon_days; i++)
      {
        sprintf(tmp, "%02d ", j);
        strcat(week[k], tmp);
        j++;
      }
      week[k][strlen(week[k]) - 1] = '\0';
    }
  }
}

//刷新窗口
void window_reloading(void)
{
  P_Window p_old_window = (P_Window)app_window_stack_get_window_by_id(g_window);
  P_Window p_window = init_watch();
  g_window = app_window_stack_replace_window(p_old_window, p_window);
}
/*时间更改*/
void watch_time_change(enum SysEventType type, void *context)
{
  if (type == SysEventTypeTimeChange && mode_switch == 4)
  {
    struct date_time dt;
    app_service_get_datetime(&dt);
    if (dt.year + dt.mon != yearpmon)
    {
      mon_days = get_mon_days(dt.year, dt.mon);
      first_day = get_first_day(dt.mday, dt.wday);
      calendar_str(first_day, mon_days);
      yearpmon = dt.year + dt.mon;
    }
    window_reloading();
  }
}

//定义选择键功能，切换模式
static void select_switch(void *context)
{
  struct date_time dt;
  app_service_get_datetime(&dt);
  if (mode_switch == 5)
  {
    mode_switch = 4;
    mon_days = get_mon_days(dt.year, dt.mon);
    first_day = get_first_day(dt.mday, dt.wday);
    calendar_str(first_day, mon_days);
  }
  else
  {
    mode_switch = 5;
    int16_t year = dt.year;
    int8_t mon = dt.mon;
    if (mon == 12)
    {
      year += 1;
      mon = 0;
    }
    mon += 1;
    mon_days = get_mon_days(year, mon);
    first_day = (dt.wday + get_mon_days(dt.year, dt.mon) - dt.mday) % 7 + 1;
    if (first_day > 6)
    {
      first_day = 0;
    }
    calendar_str(first_day, mon_days);
  }
  window_reloading();
}

/*创建并显示图层，需要坐标值，得到icon_key的数组，数组需要的参数值，P_Window*/
int32_t display_target_layer(P_Window p_window, GRect *temp_p_frame, int32_t bmp_array_name_key)
{
  GBitmap bmp_point;
  res_get_user_bitmap(bmp_array_name_key, &bmp_point);
  LayerBitmap layer_bitmap_struct_l = {bmp_point, *temp_p_frame, GAlignLeft};
  P_Layer temp_P_Layer = app_layer_create_bitmap(&layer_bitmap_struct_l);
  if (temp_P_Layer != NULL)
  {
    return app_window_add_layer(p_window, temp_P_Layer);
  }
  return 0;
}

int32_t display_target_layerText(P_Window p_window, const GRect *temp_p_frame, enum GAlign how_to_align, enum GColor color, char *str, uint8_t font_type)
{
  LayerText temp_LayerText = {0};
  temp_LayerText.text = str;
  temp_LayerText.frame = *temp_p_frame;
  temp_LayerText.alignment = how_to_align;
  temp_LayerText.font_type = font_type;
  P_Layer p_layer = app_layer_create_text(&temp_LayerText);
  if (p_layer != NULL)
  {
    app_layer_set_bg_color(p_layer, color);
    return app_window_add_layer(p_window, p_layer);
  }
  return 0;
}
//载入窗口
P_Window init_watch()
{
  char buff[30] = {0};
  P_Window p_window = app_window_create();
  int16_t i = 0;
  //时间图层
  struct date_time dt;
  app_service_get_datetime(&dt);
  GRect frame = {{5, 0}, {32, 90}};
  sprintf(buff, "%02d:%02d", dt.hour, dt.min);
  display_target_layerText(p_window, &frame, GAlignLeft, GColorWhite, buff, U_ASCII_ARIAL_30);

  //月份图层
  memset(buff, 0, sizeof(buff));
  i = dt.year;
  if (mode_switch == 5 && dt.mon == 12)
    i += 1;
  sprintf(buff, "%d", i);
  frame.origin.x = 90;
  frame.origin.y = 3;
  frame.size.h = 14;
  frame.size.w = 32;
  display_target_layerText(p_window, &frame, GAlignCenter, GColorWhite, buff, U_ASCII_ARIAL_12);

  memset(buff, 0, sizeof(buff));

  if (mode_switch == 5)
  {
    i = dt.mon + 1;
    if (i > 12)
      i = 1;
    sprintf(buff, "%02d月", i);
  }
  else
  {
    sprintf(buff, "%02d月", dt.mon);
  }
  frame.origin.y = 16;
  display_target_layerText(p_window, &frame, GAlignRight, GColorWhite, buff, U_ASCII_ARIAL_12);

  //星期背景条
  frame.origin.x = 0;
  frame.origin.y = 33;
  frame.size.h = 15;
  frame.size.w = 128;
  display_target_layer(p_window, &frame, RES_BG);
  //日历第1行
  frame.origin.x = 5;
  frame.origin.y = 51;
  frame.size.h = 12;
  frame.size.w = 116;
  display_target_layerText(p_window, &frame, GAlignRight, GColorWhite, week[1], U_ASCII_ARIAL_12);
  //日历第2到6行

  for (i = 2; i <= 6; i++)
  {
    if (strlen(week[i]) > 1)
    {
      frame.origin.y += 13;
      display_target_layerText(p_window, &frame, GAlignLeft, GColorWhite, week[i], U_ASCII_ARIAL_12);
    }
  }
  //标记当天
  if (mode_switch == 4)
  {
    memset(buff, 0, sizeof(buff));
    sprintf(buff, "%02d", dt.mday);
    frame.origin.x = 4 + 17 * dt.wday;
    frame.origin.y = 51 + 13 * ((dt.mday + first_day - 1) / 7);
    frame.size.h = 13;
    frame.size.w = 17;
    display_target_layerText(p_window, &frame, GAlignCenter, GColorBlack, buff, U_ASCII_ARIAL_12);
  }
  //按键回调
  app_window_click_subscribe(p_window, ButtonIdSelect, select_switch);
  return p_window;
}

int main()
{
  //simulator_init();
  //初始化数据
  struct date_time dt;
  app_service_get_datetime(&dt);
  yearpmon = dt.year + dt.mon;
  mon_days = get_mon_days(dt.year, dt.mon);
  first_day = get_first_day(dt.mday, dt.wday);
  calendar_str(first_day, mon_days);

  P_Window p_window = init_watch();
  g_window = app_window_stack_push(p_window);
  maibu_service_sys_event_subscribe(watch_time_change);

  //simulator_wait();
  return 0;
}